==> nono-0.4.4.tar.gz: nono-0.4.4/vm/adpcm.cpp <==
//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// ADPCM (MSM6258V)
//

#include "adpcm.h"
#include "dmac.h"
#include "mpu.h"
#include "pio.h"
#include "scheduler.h"

// グローバル参照用
ADPCMDevice *gADPCM;

// コンストラクタ
ADPCMDevice::ADPCMDevice()
	: inherited("ADPCM")
{
	event.func = ToEventCallback(&ADPCMDevice::EventCallback);
	event.Regist("ADPCM");
}

// デストラクタ
ADPCMDevice::~ADPCMDevice()
{
	gADPCM = NULL;
}

// リセット
void
ADPCMDevice::ResetHard(bool poweron)
{
	playing = false;
	dack = false;

	// ?
	clk = 8000'000;
	div = 512;
}

uint64
ADPCMDevice::Read(uint32 offset)
{
	uint8 data;

	gMPU->AddCycle(18);	// InsideOut p.135

	switch (offset) {
	 case 0:	// STAT
		data = 0x40;
		if (playing) {
			data |= STAT_PLAY;
		}
		putlog(1, "STAT -> $%02x", data);
		break;
	 case 1:	// DATA
		putlog(0, "Read $%06x (NOT IMPLEMENTED)", gMPU->GetPaddr());
		data = 0xff;
		break;
	 default:
		__unreachable();
	}

	return data;
}

uint64
ADPCMDevice::Write(uint32 offset, uint32 data)
{
	gMPU->AddCycle(22);	// InsideOut p.135

	switch (offset) {
	 case 0:	// CMD
		putlog(1, "CMD  <- $%02x", data);
		if ((data & REC_START)) {
			putlog(0, "Start recording (NOT IMPLEMENTED)");
		}
		if ((data & PLAY_START)) {
			putlog(0, "Start playing (NOT IMPLEMENTED)");
			StartPlay();
		}
		if ((data & STOP)) {
			putlog(0, "Stop (NOT IMPLEMENTED)");
			playing = false;
		}
		break;
	 case 1:	// DATA
		// 未実装だけど再生中の書き込みは来るのは分かっているので抑えておく
		if ((playing && loglevel >= 2) || playing == false) {
			putlogn("DATA <- $%02x (NOT IMPLEMENTED)", data);
		}
		break;
	 default:
		__unreachable();
	}

	return 0;
}

uint64
ADPCMDevice::Peek(uint32 offset)
{
	// XXX 未実装
	return 0xff;
}

// 再生開始
void
ADPCMDevice::StartPlay()
{
	// PPI で設定されているサンプリングレートをここで読み出す。
	// XXX パンは未対応
	switch (gPPI->GetADPCMRate()) {
	 case 0:
		div = 1024;
		break;
	 case 1:
	 case 3:		// %11 は %01 と同じになる (Inside p295)
		div = 768;
		break;
	 case 2:
		div = 512;
		break;
	 default:
		__unreachable();
	}

	playing = true;
	gDMAC->AssertREQ(3);

	event.time = div / clk;
	gScheduler->StartEvent(event);
}

// イベント
void
ADPCMDevice::EventCallback(Event& ev)
{
	// XXX 適当
	if (playing) {
		gDMAC->AssertREQ(3);
	}
}

// DACK 信号をアサートする (DMAC から呼ばれる)
void
ADPCMDevice::AssertDACK(bool tc_)
{
	dack = true;
	// DRQ を下げる
	gDMAC->NegateREQ(0);
}

// DACK 信号をネゲートする (DMAC から呼ばれる)
void
ADPCMDevice::NegateDACK()
{
	dack = false;
}

// クロック切り替え。OPM から呼ばれる。
// CT1 = %0 で 8MHz、%1 で 4MHz。
void
ADPCMDevice::SetClock(bool ct1)
{
	if (ct1) {
		clk = 4000'000;
	} else {
		clk = 8000'000;
	}
}

==> nono-0.6.3.tar.gz: nono-0.6.3/vm/adpcm.cpp <==
//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// ADPCM (MSM6258V)
//

#include "adpcm.h"
#include "dmac.h"
#include "mpu.h"
#include "pio.h"
#include "scheduler.h"

// コンストラクタ
ADPCMDevice::ADPCMDevice()
	: inherited(OBJ_ADPCM)
{
	event.func = ToEventCallback(&ADPCMDevice::EventCallback);
	event.Regist("ADPCM");
}

// デストラクタ
ADPCMDevice::~ADPCMDevice()
{
}

// 初期化
bool
ADPCMDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	dmac = GetDMACDevice();
	ppi = GetPPIDevice();

	return true;
}

// リセット
void
ADPCMDevice::ResetHard(bool poweron)
{
	playing = false;
	dack = false;

	// ?
	clk = 8000'000;
	div = 512;
}

uint64
ADPCMDevice::Read(uint32 offset)
{
	uint8 data;

	mpu->AddCycle(18);	// InsideOut p.135

	switch (offset) {
	 case 0:	// STAT
		data = 0x40;
		if (playing) {
			data |= STAT_PLAY;
		}
		putlog(1, "STAT -> $%02x", data);
		break;
	 case 1:	// DATA
		putlog(0, "Read $%06x (NOT IMPLEMENTED)", mpu->GetPaddr());
		data = 0xff;
		break;
	 default:
		VMPANIC("corrupted offset=%d", offset);
	}

	return data;
}

uint64
ADPCMDevice::Write(uint32 offset, uint32 data)
{
	mpu->AddCycle(22);	// InsideOut p.135

	switch (offset) {
	 case 0:	// CMD
		putlog(1, "CMD  <- $%02x", data);
		if ((data & REC_START)) {
			putlog(0, "Start recording (NOT IMPLEMENTED)");
		}
		if ((data & PLAY_START)) {
			putlog(0, "Start playing (NOT IMPLEMENTED)");
			StartPlay();
		}
		if ((data & STOP)) {
			putlog(0, "Stop (NOT IMPLEMENTED)");
			playing = false;
		}
		break;
	 case 1:	// DATA
		// 未実装だけど再生中の書き込みは来るのは分かっているので抑えておく
		if ((playing && loglevel >= 2) || playing == false) {
			putlogn("DATA <- $%02x (NOT IMPLEMENTED)", data);
		}
		break;
	 default:
		VMPANIC("corrupted offset=%d", offset);
	}

	return 0;
}

uint64
ADPCMDevice::Peek(uint32 offset)
{
	// XXX 未実装
	return 0xff;
}

// 再生開始
void
ADPCMDevice::StartPlay()
{
	// PPI で設定されているサンプリングレートをここで読み出す。
	// XXX パンは未対応
	int rate = ppi->GetADPCMRate();
	switch (rate) {
	 case 0:
		div = 1024;
		break;
	 case 1:
	 case 3:		// %11 は %01 と同じになる (Inside p295)
		div = 768;
		break;
	 case 2:
		div = 512;
		break;
	 default:
		VMPANIC("corrupted ADPCM rate=%d", rate);
	}

	playing = true;
	dmac->AssertREQ(3);

	event.time = div / clk;
	scheduler->StartEvent(event);
}

// イベント
void
ADPCMDevice::EventCallback(Event& ev)
{
	// XXX 適当
	if (playing) {
		dmac->AssertREQ(3);
	}
}

// DACK 信号をアサートする (DMAC から呼ばれる)
void
ADPCMDevice::AssertDACK(bool tc_)
{
	dack = true;
	// DRQ を下げる
	dmac->NegateREQ(0);
}

// DACK 信号をネゲートする (DMAC から呼ばれる)
void
ADPCMDevice::NegateDACK()
{
	dack = false;
}

// クロック切り替え。OPM から呼ばれる。
// CT1 = %0 で 8MHz、%1 で 4MHz。
void
ADPCMDevice::SetClock(bool ct1)
{
	if (ct1) {
		clk = 4000'000;
	} else {
		clk = 8000'000;
	}
}
